/****************************************************************************/
/****************************************************************************/
/**																		   **/
/**					   Calcul de l'nergie sur une batterie				   **/
/**																		   **/
/****************************************************************************/
/****************************************************************************/
/**																		   **/
/**						        Julien VILLEMEJANE						   **/
/**																		   **/
/****************************************************************************/
/****************************************************************************/
/**  	                   ENS Cachan - Dpartement EEA					   **/
/****************************************************************************/
/****************************************************************************/


		// Port 0 - Bit 5 --> Sortie analogique Vref=2,5V
 
	    // Port 1 - Bit 0 --> Signal rechargement
    	// Port 1 - Bit 1 --> Signal batterie dcharge
    	// Port 1 - Bit 2 --< Entre de rinitialisation
    	// Port 1 - Bit 3 --> Signal allum/teint
		// Port 1 - Bit 4 --< Entre de rinitialisation (test)

    	// Port 2 - Bit 1 --< Entre analogique U
    	// Port 2 - Bit 2 --< Entre analogique I


#include "m8c.h"       			// part specific constants and macros
#include "PSoCAPI.h"    		// PSoC API definitions for all User Modules

// Dclaration des variables
/* Variables "variables" */
long U,I,Ucalc;
long E,Ep;							// Energie discrte
long En;							// Energie "instantane" discrte
char Es[4];							// Energie convertie en char	
long cpt=0,cpts=0;					// compteur pour l'initialisation et la sauvegarde

/* Variables "fixes" */
long Uref=2200;						// tension de rfrence (correspond  10,3 V environ)
long Ubas=2000;						// valeur critique de dcharge de la batterie (environ U=9V)
long Eref=100000000;				// permet de tenir 4-5 min aprs l'envoi du signal de rechargement
long k=4;							// coefficient multiplicateur pour le calcul de E
long f=300;							// frequence pour le calcul de E : E = U.I/(k*f)


/************************************/
/* Fonction d'acquisition de U et I */
/************************************/
void Acquis(){
	PRT1DR ^= 0x20;									// Alternance sur le bit 5 du port 1
													// permet de mesurer la frquence de conversion
	ADCINC12_U_GetSamples(1);
	ADCINC12_I_GetSamples(1);
	
	while(ADCINC12_U_fIsDataAvailable() == 0);		// Attente de rcupration de U
	U=(ADCINC12_U_iGetData()+2048);					// Rcupration de U
	ADCINC12_U_ClearFlag();

	while(ADCINC12_I_fIsDataAvailable() == 0);		// Attente de rcupration de I
	I=2*ADCINC12_I_iGetData();						// Rcupration de I
	ADCINC12_I_ClearFlag();
	
	Ucalc=U/16;										// Calcul de Ucalc
	DAC8_REF_WriteBlind(Ucalc);						// Envoi sur la sortie du CNA
	
}


/************************************/
/* Fonction de translation E --> Es */
/*  		long  -> char			*/
/*									*/
/* 		Sauvegarde des donnes 		*/
/*	  toutes les 30 sec en Flash	*/
/************************************/
void saveE(){
	if(cpts>9000){							// procedure de memorisation toute les 30s environ
		// MSB
		Ep = E & 0xFF000000;				// Application d'un masque sur les bits de poids fort
		Ep=Ep>>24;							// Decalage a droite de 24 bits
		Es[0]=(unsigned char)Ep;			// Memorisation
		// 3e octet
		Ep = E & 0x00FF0000;				// Application d'un masque sur les 8 bits suivants
		Ep=Ep>>16;							// Decalage a droite de 16 bits
		Es[1]=(unsigned char)Ep;			// Memorisation
		// 2e octet
		Ep = E & 0x0000FF00;				// Application d'un masque sur les 8 bits suivants
		Ep=Ep>>8;							// Decalage a droite de 8 bits
		Es[2]=(unsigned char)Ep;			// Memorisation
		// LSB	
		Ep = E & 0x000000FF;				// Application d'un masque sur les bits de poids faible
		Es[3]=(unsigned char)Ep;			// Memorisation
		// Ecriture en Flash
		E2PROM_E_bE2Write(0,Es,4,25);
		cpts=0;
	}
	else{cpts++;}
}


/************************************/
/* Fonction de translation Es --> E */
/*  		char  -> long			*/
/************************************/
void readE(){
	// Lecture sur Flash
	E2PROM_E_E2Read(0,Es,4);
	//MSB
	Ep=(unsigned long)Es[0];				// Acquisition des 8 premiers bits memoriss
	Ep=Ep<<8;								// Decalage a gauche de 8 bits
	// 3e octet
	Ep+=(unsigned long)Es[1];				// Acquisition des 8 bits suivants
	Ep=Ep<<8;								// Decalage a gauche de 8 bits
	// 2e octet
	Ep+=(unsigned long)Es[2];				// Acquisition des 8 bits suivants
	Ep=Ep<<8;								// Decalage a gauche de 8 bits
	// LSB
	Ep+=(unsigned long)Es[3];				// Acquisition des 8 derniers bits memoriss
	E=Ep;									// Stockage de la valeur lue dans E
}


/********************************/
/* Fonction de rinitialisation */
/********************************/
// Fonction permettant de rinitialiser la valeur de E
void EraseE(){
	if((PRT1DR & 0x04)==0){
		E=Eref;
		PRT1DR |= 0x04;
		saveE();
	}

// Fonction supplmentaire pour les tests, qui rinitialise E  2*Eref
	if((PRT1DR & 0x10)==0){
		E=2*Eref;
		PRT1DR |= 0x10;
		PRT1DR &= ~0x01; PRT1DR &= ~0x02;
		saveE();
	}
}


/*****************************************/
/* Fonction d'initialisation des modules */
/*****************************************/
void Init(){
	// Initialisation PSoC
    M8C_EnableGInt;
    
    
    // Dmarrage des CAN et CNA
    ADCINC12_I_Start(ADCINC12_I_HIGHPOWER);
    ADCINC12_U_Start(ADCINC12_U_HIGHPOWER);
    DAC8_REF_Start(DAC8_REF_HIGHPOWER);  
    
    
    // Dmarrage des blocs analogiques
    PGA_U_Start(ADCINC12_I_HIGHPOWER);  
    INSAMP_1_Start(ADCINC12_I_HIGHPOWER);
    
    
    // EEPROM - Rcupration de la valeur de E prcdemment stocke
    E2PROM_E_Start();
    readE();
    
    
    // Connexion des ports numriques
	    // Port 1 - Bit 0 --> Signal rechargement
    	// Port 1 - Bit 1 --> Signal batterie dcharge
    	// Port 1 - Bit 2 --< Signal de rinitialisation
    	// Port 1 - Bit 3 --> Signal allum/teint
    PRT1DR &= ~0x0B;  		// mise  zro des bits utiliss du port 1
    PRT1DR |= 0x04;			// mise  un du bit 2 du port 1
    PRT1DR |= 0x10;			// mise  un du bit 4 du port 1
    
        
	// Tempo d'initialisation pour la mise sous tension
	while(cpt<100000){cpt++;}
	PRT1DR |= 0x08;								// Calculateur prt
	
	// Rcupration de U et I (au dmarrage)
	Acquis();
}


/**********************/
/* Boucle de Decharge */
/**********************/
// Si I est ngatif, alors on rentre dans cette boucle de Decharge.
// I tant ngatif, le rsultat E l'est aussi. Le compteur d'nergie E dcrmente.
// S'il atteind une valeur limite Eref, on envoie alors le signal de rechargement au robot.

void Decharge() {
	EraseE();

		if((E>Eref)&&(U>Uref))PRT1DR &= ~0x01;		// Mise  1 du bit 0 du port 1 - Pas besoin de recharger
		Acquis();									// Acquisition de U et I  une frquence de 300 Hz		
						
		if(U>Uref){									// Test si U est suprieure  une tension de rfrence
			En=U*I/(k*f);							// si c'est le cas, calcul de P
			if(E>En){E+=En;}				
			else {PRT1DR |= 0x02; E=0;}				// Erreur batterie dcharge
			saveE();
			
		}  	 	
									
		else{										// sinon
			for(cpt=0;cpt<300;cpt++){				// Attente d'une seconde pour retester U
													// permet de savoir si une chute de tension est momentane
													// (due  un appel de courant) ou si la batterie est dechargee
				Acquis();
				En=U*I/(k*f);
				if(E>En)E+=En;
				else {PRT1DR |= 0x02; E=0;}			// Erreur batterie dcharge
				saveE();
				if(E<Eref){							// Test sur l'nergie
					PRT1DR |= 0x01;					// On envoie l'ordre au robot d'aller se recharger
				}
			}
			if(U<Uref){								// Test sur U
				PRT1DR |= 0x01;						// On envoie l'ordre au robot d'aller se recharger
    		}
    		if(U<Ubas){	
    			PRT1DR |= 0x02;						// Erreur batterie dcharge : ordre d'arret d'urgence
    		}
		} 
					 				
		if(E<Eref){									// Test sur l'nergie	
			PRT1DR |= 0x01;							// On envoie l'ordre au robot d'aller se recharger
		}
	
}



/********************/
/* Boucle de Charge */
/********************/
// Si I est positif, alors on rentre dans cette boucle de Charge.
// I tant positif, le rsultat E l'est aussi. Le compteur d'nergie E s'incrmente.

void Charge() {

	EraseE();
	if(E>Eref) { PRT1DR &= ~0x01; PRT1DR &= ~0x02;}		// Mise  1 du bit 0 du port 1
			
	Acquis();											// Acquisition de U et I  une frquence de 300 Hz

	En=U*I/(k*f);										// Calcul de E
	E+=En;
	
	saveE();											// Memorisation de la valeur de l'energie
}




/***********************/
/*					   */
/* Programme principal */
/*					   */
/***********************/
void main()
{	
	Init();						// Initialisation
	while(1){					// Gestion de l'utilisation et de la charge
		while(I<=0) Decharge();	
		while(I>0) Charge();
		
	}

}